#!/bin/bash
#
#===============================================================================
#
# Copia de seguridad incremental con rsync
# 
#===============================================================================
#

#*******************************************************************************
# rutas a modificar

dir_origen=/RUTA/A/datos # sin la / al final de la ruta para que en incr
                         # guarde todo en directorios
dir_espejo=/RUTA/A/copia_seguridad
dir_incr=/RUTA/A/copia_seguridad/incr

#*******************************************************************************

if [ $# -ge 1 ]
then
     echo "Uso: ${0##*/}"
     exit 1
fi

# comprobar si hay espacio en el disco. Por lo menos 100 MiB=102400 KiB

echo "Comprobando si hay espacio en destino..."

umount /home/a/.gvfs >/dev/null 2>&1 # esto es por un error de Ubuntu 10.04
origen=$(nice -n19 ionice -c3 \
du -s --exclude=".local/share/Trash" "${dir_origen}" | awk '{ print $1 }' )

espejo=$(nice -n19 ionice -c3 du -s  "${dir_espejo}" | awk '{ print $1 }' )

# No es, como en un principio podría parecer, origen-espejo, sino 
# origen+espejo, porque podría ocurrir que modificásemos todos los ficheros
# de origen cambiando solo un cáracter y rsync enviaría todos los ficheros de 
# destino a incr, puesto que han sido modificados, y luego copiaría todos los ficheros
# de origen a espejo. Y si hubiéramos añadido o borrado ficheros también 
# funcionaría correctamente el cálculo origen+espejo.

datos=$((origen + espejo))

espacio=$(df "${dir_espejo}" | awk '{ print $4 }' | tail -n 1)
# también con: df "${dir_espejo}" | tr -s ' ' | cut -d ' ' -f4 | tail -n1

# averiguar cuántas copias hay hechas
copias=$(ls "${dir_incr}" | wc -l)

while [ $((espacio - datos)) -lt 102400 ] # 102400 KiB = 100 MiB
do
# si solo hay una copia incremental salir del programa
     if [ $copias -eq 1 ]
     then
        mail -s "${0##*/}: Error, espacio de disco insuficiente para realizar la copia" a </dev/null >/dev/null 2>&1 &
        echo "${0##*/}: Error, espacio de disco insuficiente para realizar la copia"
        exit 1
     fi
# si hay más de una copia borrar la última y comprobar de nuevo
     if [ $copias -gt 1 ]
     then
         nice -n19 ionice -c3 rm -vr "$dir_incr"/$(ls "$dir_incr" -tr | head -n 1)
         copias=$((copias-1))
         espacio=$(df "$dir_espejo" | awk '{ print $4 }' | tail -n 1)
     fi
done

fecha="rsync-$(date +%Y-%m-%d-%H-%M-%S)" &&  mkdir "${dir_incr}/${fecha}"

echo -e "Mensajes de salida y de error de rsync en el directorio:\n${dir_incr}/${fecha}"
echo "Sincronizando directorios..."

nice -n 19 ionice -c 3 rsync -avb --exclude=".local/share/Trash"          \
--delete                                                                  \
--backup-dir="${dir_incr}/${fecha}"                                       \
"${dir_origen}" "${dir_espejo}"                                           \
1>"${dir_incr}/${fecha}/salida.txt" 2>"${dir_incr}/${fecha}/error.txt"
exit 0
